@@ -29,7 +29,7 @@
     \sa wc_AesSetKeyDirect
     \sa wc_AesSetIV
 */
-WOLFSSL_API int  wc_AesSetKey(Aes* aes, const byte* key, word32 len,
+int  wc_AesSetKey(Aes* aes, const byte* key, word32 len,
                               const byte* iv, int dir);
 
 /*!
@@ -59,7 +59,7 @@ WOLFSSL_API int  wc_AesSetKey(Aes* aes, const byte* key, word32 len,
     \sa wc_AesSetKeyDirect
     \sa wc_AesSetKey
 */
-WOLFSSL_API int  wc_AesSetIV(Aes* aes, const byte* iv);
+int  wc_AesSetIV(Aes* aes, const byte* iv);
 
 /*!
     \ingroup AES
@@ -107,7 +107,7 @@ WOLFSSL_API int  wc_AesSetIV(Aes* aes, const byte* iv);
     \sa wc_AesSetIV
     \sa wc_AesCbcDecrypt
 */
-WOLFSSL_API int  wc_AesCbcEncrypt(Aes* aes, byte* out,
+int  wc_AesCbcEncrypt(Aes* aes, byte* out,
                                   const byte* in, word32 sz);
 
 /*!
@@ -156,7 +156,7 @@ WOLFSSL_API int  wc_AesCbcEncrypt(Aes* aes, byte* out,
     \sa wc_AesSetKey
     \sa wc_AesCbcEncrypt
 */
-WOLFSSL_API int  wc_AesCbcDecrypt(Aes* aes, byte* out,
+int  wc_AesCbcDecrypt(Aes* aes, byte* out,
                                   const byte* in, word32 sz);
 
 /*!
@@ -201,18 +201,18 @@ WOLFSSL_API int  wc_AesCbcDecrypt(Aes* aes, byte* out,
 
     \sa wc_AesSetKey
 */
-WOLFSSL_API int wc_AesCtrEncrypt(Aes* aes, byte* out,
+int wc_AesCtrEncrypt(Aes* aes, byte* out,
                                    const byte* in, word32 sz);
 
 /*!
     \ingroup AES
     \brief This function is a one-block encrypt of the input block, in, into
-    the output block, out. It uses the key and iv (initialization vector)
-    of the provided AES structure, which should be initialized with
-    wc_AesSetKey before calling this function. It is only enabled if the
-    configure option WOLFSSL_AES_DIRECT is enabled. __Warning:__ In nearly all
-    use cases ECB mode is considered to be less secure. Please avoid using ECB
-    API’s directly whenever possible
+    the output block, out. It uses the key of the provided AES structure, which
+    should be initialized with wc_AesSetKey before calling this function.
+    wc_AesSetKey should have been called with the iv set to NULL. This is only
+    enabled if the configure option WOLFSSL_AES_DIRECT is enabled. __Warning:__
+    In nearly all use cases ECB mode is considered to be less secure. Please
+    avoid using ECB API’s directly whenever possible.
 
     \return int integer values corresponding to wolfSSL error or success
     status
@@ -235,18 +235,17 @@ WOLFSSL_API int wc_AesCtrEncrypt(Aes* aes, byte* out,
     \sa wc_AesDecryptDirect
     \sa wc_AesSetKeyDirect
 */
-WOLFSSL_API int wc_AesEncryptDirect(Aes* aes, byte* out, const byte* in);
+int wc_AesEncryptDirect(Aes* aes, byte* out, const byte* in);
 
 /*!
     \ingroup AES
     \brief This function is a one-block decrypt of the input block, in, into
-    the output block, out. It uses the key and iv (initialization vector) of
-    the provided AES structure, which should be initialized with wc_AesSetKey
-    before calling this function. It is only enabled if the configure option
-    WOLFSSL_AES_DIRECT is enabled, and there is support for direct AES
-    encryption on the system in question. __Warning:__ In nearly all use cases
-    ECB mode is considered to be less secure. Please avoid using ECB API’s
-    directly whenever possible
+    the output block, out. It uses the key of the provided AES structure, which
+    should be initialized with wc_AesSetKey before calling this function.
+    wc_AesSetKey should have been called with the iv set to NULL. This is only
+    enabled if the configure option WOLFSSL_AES_DIRECT is enabled. __Warning:__
+    In nearly all use cases ECB mode is considered to be less secure. Please
+    avoid using ECB API’s directly whenever possible.
 
     \return int integer values corresponding to wolfSSL error or success
     status
@@ -270,7 +269,7 @@ WOLFSSL_API int wc_AesEncryptDirect(Aes* aes, byte* out, const byte* in);
     \sa wc_AesEncryptDirect
     \sa wc_AesSetKeyDirect
  */
-WOLFSSL_API int wc_AesDecryptDirect(Aes* aes, byte* out, const byte* in);
+int wc_AesDecryptDirect(Aes* aes, byte* out, const byte* in);
 
 /*!
     \ingroup AES
@@ -310,7 +309,7 @@ WOLFSSL_API int wc_AesDecryptDirect(Aes* aes, byte* out, const byte* in);
     \sa wc_AesDecryptDirect
     \sa wc_AesSetKey
 */
-WOLFSSL_API int  wc_AesSetKeyDirect(Aes* aes, const byte* key, word32 len,
+int  wc_AesSetKeyDirect(Aes* aes, const byte* key, word32 len,
                                 const byte* iv, int dir);
 
 /*!
@@ -340,7 +339,7 @@ WOLFSSL_API int  wc_AesSetKeyDirect(Aes* aes, const byte* key, word32 len,
     \sa wc_AesGcmEncrypt
     \sa wc_AesGcmDecrypt
 */
-WOLFSSL_API int  wc_AesGcmSetKey(Aes* aes, const byte* key, word32 len);
+int  wc_AesGcmSetKey(Aes* aes, const byte* key, word32 len);
 
 /*!
     \ingroup AES
@@ -385,7 +384,7 @@ WOLFSSL_API int  wc_AesGcmSetKey(Aes* aes, const byte* key, word32 len);
     \sa wc_AesGcmSetKey
     \sa wc_AesGcmDecrypt
 */
-WOLFSSL_API int  wc_AesGcmEncrypt(Aes* aes, byte* out,
+int  wc_AesGcmEncrypt(Aes* aes, byte* out,
                                    const byte* in, word32 sz,
                                    const byte* iv, word32 ivSz,
                                    byte* authTag, word32 authTagSz,
@@ -434,7 +433,7 @@ WOLFSSL_API int  wc_AesGcmEncrypt(Aes* aes, byte* out,
     \sa wc_AesGcmSetKey
     \sa wc_AesGcmEncrypt
 */
-WOLFSSL_API int  wc_AesGcmDecrypt(Aes* aes, byte* out,
+int  wc_AesGcmDecrypt(Aes* aes, byte* out,
                                    const byte* in, word32 sz,
                                    const byte* iv, word32 ivSz,
                                    const byte* authTag, word32 authTagSz,
@@ -456,12 +455,14 @@ WOLFSSL_API int  wc_AesGcmDecrypt(Aes* aes, byte* out,
     \code
     Gmac gmac;
     key[] = { some 16, 24, or 32 byte length key };
+    wc_AesInit(gmac.aes, HEAP_HINT, INVALID_DEVID); // Make sure devId updated
     wc_GmacSetKey(&gmac, key, sizeof(key));
     \endcode
 
     \sa wc_GmacUpdate
+    \sa wc_AesInit
 */
-WOLFSSL_API int wc_GmacSetKey(Gmac* gmac, const byte* key, word32 len);
+int wc_GmacSetKey(Gmac* gmac, const byte* key, word32 len);
 
 /*!
     \ingroup AES
@@ -487,6 +488,7 @@ WOLFSSL_API int wc_GmacSetKey(Gmac* gmac, const byte* key, word32 len);
     key[] = { some 16, 24, or 32 byte length key };
     iv[] = { some 16 byte length iv };
 
+    wc_AesInit(gmac.aes, HEAP_HINT, INVALID_DEVID); // Make sure devId updated
     wc_GmacSetKey(&gmac, key, sizeof(key));
     authIn[] = { some 16 byte authentication input };
     tag[AES_BLOCK_SIZE]; // will store authentication code
@@ -496,8 +498,9 @@ WOLFSSL_API int wc_GmacSetKey(Gmac* gmac, const byte* key, word32 len);
     \endcode
 
     \sa wc_GmacSetKey
+    \sa wc_AesInit
 */
-WOLFSSL_API int wc_GmacUpdate(Gmac* gmac, const byte* iv, word32 ivSz,
+int wc_GmacUpdate(Gmac* gmac, const byte* iv, word32 ivSz,
                                const byte* authIn, word32 authInSz,
                                byte* authTag, word32 authTagSz);
 
@@ -524,7 +527,7 @@ WOLFSSL_API int wc_GmacUpdate(Gmac* gmac, const byte* iv, word32 ivSz,
     \sa wc_AesCcmEncrypt
     \sa wc_AesCcmDecrypt
 */
-WOLFSSL_API int  wc_AesCcmSetKey(Aes* aes, const byte* key, word32 keySz);
+int  wc_AesCcmSetKey(Aes* aes, const byte* key, word32 keySz);
 
 /*!
     \ingroup AES
@@ -569,7 +572,7 @@ WOLFSSL_API int  wc_AesCcmSetKey(Aes* aes, const byte* key, word32 keySz);
     \sa wc_AesCcmSetKey
     \sa wc_AesCcmDecrypt
 */
-WOLFSSL_API int  wc_AesCcmEncrypt(Aes* aes, byte* out,
+int  wc_AesCcmEncrypt(Aes* aes, byte* out,
                                    const byte* in, word32 inSz,
                                    const byte* nonce, word32 nonceSz,
                                    byte* authTag, word32 authTagSz,
@@ -624,7 +627,7 @@ WOLFSSL_API int  wc_AesCcmEncrypt(Aes* aes, byte* out,
     \sa wc_AesCcmSetKey
     \sa wc_AesCcmEncrypt
 */
-WOLFSSL_API int  wc_AesCcmDecrypt(Aes* aes, byte* out,
+int  wc_AesCcmDecrypt(Aes* aes, byte* out,
                                    const byte* in, word32 inSz,
                                    const byte* nonce, word32 nonceSz,
                                    const byte* authTag, word32 authTagSz,
@@ -662,7 +665,7 @@ WOLFSSL_API int  wc_AesCcmDecrypt(Aes* aes, byte* out,
     \sa wc_AesXtsDecrypt
     \sa wc_AesXtsFree
 */
-WOLFSSL_API int wc_AesXtsSetKey(XtsAes* aes, const byte* key,
+int wc_AesXtsSetKey(XtsAes* aes, const byte* key,
          word32 len, int dir, void* heap, int devId);
 
 /*!
@@ -701,7 +704,7 @@ WOLFSSL_API int wc_AesXtsSetKey(XtsAes* aes, const byte* key,
     \sa wc_AesXtsSetKey
     \sa wc_AesXtsFree
 */
-WOLFSSL_API int wc_AesXtsEncryptSector(XtsAes* aes, byte* out,
+int wc_AesXtsEncryptSector(XtsAes* aes, byte* out,
          const byte* in, word32 sz, word64 sector);
 
 /*!
@@ -740,7 +743,7 @@ WOLFSSL_API int wc_AesXtsEncryptSector(XtsAes* aes, byte* out,
     \sa wc_AesXtsSetKey
     \sa wc_AesXtsFree
 */
-WOLFSSL_API int wc_AesXtsDecryptSector(XtsAes* aes, byte* out,
+int wc_AesXtsDecryptSector(XtsAes* aes, byte* out,
          const byte* in, word32 sz, word64 sector);
 
 /*!
@@ -780,7 +783,7 @@ WOLFSSL_API int wc_AesXtsDecryptSector(XtsAes* aes, byte* out,
     \sa wc_AesXtsSetKey
     \sa wc_AesXtsFree
 */
-WOLFSSL_API int wc_AesXtsEncrypt(XtsAes* aes, byte* out,
+int wc_AesXtsEncrypt(XtsAes* aes, byte* out,
          const byte* in, word32 sz, const byte* i, word32 iSz);
 
 /*!
@@ -819,7 +822,7 @@ WOLFSSL_API int wc_AesXtsEncrypt(XtsAes* aes, byte* out,
     \sa wc_AesXtsSetKey
     \sa wc_AesXtsFree
 */
-WOLFSSL_API int wc_AesXtsDecrypt(XtsAes* aes, byte* out,
+int wc_AesXtsDecrypt(XtsAes* aes, byte* out,
         const byte* in, word32 sz, const byte* i, word32 iSz);
 
 /*!
@@ -846,7 +849,7 @@ WOLFSSL_API int wc_AesXtsDecrypt(XtsAes* aes, byte* out,
     \sa wc_AesXtsDecrypt
     \sa wc_AesXtsSetKey
 */
-WOLFSSL_API int wc_AesXtsFree(XtsAes* aes);
+int wc_AesXtsFree(XtsAes* aes);
 
 
 /*!
@@ -873,7 +876,7 @@ WOLFSSL_API int wc_AesXtsFree(XtsAes* aes);
     \sa wc_AesSetKey
     \sa wc_AesSetIV
 */
-WOLFSSL_API int  wc_AesInit(Aes*, void*, int);
+int  wc_AesInit(Aes* aes, void* heap, int devId);
 
 /*!
     \ingroup AES
@@ -905,7 +908,7 @@ WOLFSSL_API int  wc_AesInit(Aes*, void*, int);
     \sa wc_AesCfbDecrypt
     \sa wc_AesSetKey
 */
-WOLFSSL_API int wc_AesCfbEncrypt(Aes* aes, byte* out, const byte* in, word32 sz);
+int wc_AesCfbEncrypt(Aes* aes, byte* out, const byte* in, word32 sz);
 
 /*!
     \ingroup AES
@@ -937,7 +940,7 @@ WOLFSSL_API int wc_AesCfbEncrypt(Aes* aes, byte* out, const byte* in, word32 sz)
     \sa wc_AesCfbEncrypt
     \sa wc_AesSetKey
 */
-WOLFSSL_API int wc_AesCfbDecrypt(Aes* aes, byte* out, const byte* in, word32 sz);
+int wc_AesCfbDecrypt(Aes* aes, byte* out, const byte* in, word32 sz);
 
 /*!
     \ingroup AES
@@ -981,7 +984,7 @@ WOLFSSL_API int wc_AesCfbDecrypt(Aes* aes, byte* out, const byte* in, word32 sz)
     \sa wc_AesSivDecrypt
 */
 
-WOLFSSL_API
+
 int wc_AesSivEncrypt(const byte* key, word32 keySz, const byte* assoc,
                      word32 assocSz, const byte* nonce, word32 nonceSz,
                      const byte* in, word32 inSz, byte* siv, byte* out);
@@ -1028,7 +1031,7 @@ int wc_AesSivEncrypt(const byte* key, word32 keySz, const byte* assoc,
 
     \sa wc_AesSivEncrypt
 */
-WOLFSSL_API
+
 int wc_AesSivDecrypt(const byte* key, word32 keySz, const byte* assoc,
                      word32 assocSz, const byte* nonce, word32 nonceSz,
                      const byte* in, word32 inSz, byte* siv, byte* out);
